package no.ntnu.idatt1002.demo.data.Budget;

import no.ntnu.idatt1002.demo.data.Economics.*;

import java.io.*;

/**
 * FileHandling is a class for writing and reading
 * Budgets to and from a file.
 *
 * @author andreas
 */

public class FileHandlingBudget {
    private static final String filePath = "src/main/resources/Budget";
    private static final String fileType = ".budget";

    private static final String path = "src/main/resources/";
    private static final String type = ".register";
    private static final String maxAmount = "maxAmount=";
    private static final String budgetAmount = "budgetAmount=";
    private static final String budgetCategory = "budgetCategory=";
    private static final String budgetDescription = "budgetDescription=";
    public static String id = "";

    /**
     * Method for writing (adding) a budget to a file.
     *
     * @param generalBudget the budget you want to write to a file.
     * @param fileTitle the name of the file you want to check
     * @throws IOException if an input or output exception occurred.
     */
    public static void writeGeneralBudgetToFile(String fileTitle, GeneralBudget generalBudget) throws IOException {
        try (BufferedWriter bw = new BufferedWriter(new FileWriter(path + readSelectedBudget() + "/" + fileTitle + fileType))) {
        //try (BufferedWriter bw = new BufferedWriter(new FileWriter(filePath + fileTitle + fileType))) {
            bw.write(generalBudget.toString());
        } catch (IOException ex) {
            throw new IOException("Error writing to file: " + fileTitle);
        }
    }

    /**
     * Method for checking if a .budget file is empty.
     *
     * @param fileTitle the name of the file you want to check
     * @return true or false depending on if file is empty.
     * @throws IOException if an input or output exception occurred.
     */
    public static boolean isEmpty(String fileTitle) throws IOException {
        /*try (BufferedReader br = new BufferedReader(new FileReader(filePath + fileTitle + fileType))) {
            return br.readLine() == null;
        }*/
        BufferedReader br = new BufferedReader(new FileReader(path + readSelectedBudget() + "/" + fileTitle + fileType));
        System.out.println("Trying budget isEmpty in: " + path + readSelectedBudget() + "/" + fileTitle + fileType);
        return br.readLine() == null;
    }


    /**
     * Method for checking if a .budget file is new (no categories)
     * or old (has budget categories)
     * @param fileTitle The name of the file
     * @return True, if the budget is new. Else, returns false
     * @throws IOException
     */
    public static boolean isNewBudget(String fileTitle) throws IOException {
        try (BufferedReader br = new BufferedReader(
            new FileReader(path + readSelectedBudget() + "/" + fileTitle + fileType))) {
            //new FileReader(filePath + fileTitle + fileType))) {

            for (int i = 0; i < 2; ++i) {
                br.readLine();
            }

            if (br.readLine() == null) {
                System.out.println("Is new budget - no max amount");
                return true;
            }
        }
        System.out.println("Is 'old' budget - has max amount");
        return false;
    }

    public static void writeBudgetRegisterToArchive(BudgetRegister budgetNames) throws IOException {
        try (BufferedWriter bw = new BufferedWriter(new FileWriter(System.getProperty("user.dir") + "/" + path + "Archive.txt"))) {
            System.out.println(System.getProperty("user.dir") + "/" + path + "Archive.txt");
            bw.write(budgetNames.toString());
        } catch (IOException ioe) {
            ioe.printStackTrace();
            throw new IOException("Could not write to file: Archive.txt");
        }
    }

    public static boolean isBudgetRegisterEmpty() throws IOException {
        try (BufferedReader br = new BufferedReader(new FileReader(System.getProperty("user.dir") + "/src/main/resources/"
            + "Archive.txt"))) {
            return br.readLine() == null;
        }
    }
    public static BudgetRegister readBudgetArchive(String fileTitle) throws IOException {
        BufferedReader br;
        BudgetRegister budgetRegister = null;
        String budgetName;

        try {
            br = new BufferedReader(
                new FileReader(System.getProperty("user.dir") + "/" + path
                    + "Archive.txt"));
        } catch (IOException ioe) {
            throw new IOException("File does not exist", ioe);
        }

        String line;
        String nextLine = br.readLine();
        while ((line = nextLine) != null) {
            System.out.println(line);
            nextLine = br.readLine();
            budgetName = line;

            if(budgetRegister == null){
                budgetRegister = new BudgetRegister();
            }
            budgetRegister.addBudgetName(budgetName);
        }
        return budgetRegister;
    }
    public static boolean isSelectedBudgetEmpty(String fileTitle) throws IOException {
        try (BufferedReader br = new BufferedReader(new FileReader(System.getProperty("user.dir") + "src/main/resources/"
            + "CurrentFile.txt"))) {
            return br.readLine() == null;
        }
    }

    public static void writeSavingsToFile() {

    }
    public static void writeMaxAmountToFile(String fileDestination, String maxAmount) throws IOException {
        try (BufferedWriter bw = new BufferedWriter(new FileWriter(System.getProperty("user.dir") + "/src/main/resources/"
            + fileDestination + "/Budget.budget"))) {
            bw.write("maxAmount=" + maxAmount);
            System.out.println("Max amount is...");
        } catch (IOException ex) {
            throw new IOException("Error writing to file: " + fileDestination);
        }
    }

    public static String readSelectedBudget() throws IOException {
        BufferedReader br;
        try {
            System.out.print("Reading current file...");
            FileReader fileReader = new FileReader(System.getProperty("user.dir") + "/src/main/resources/CurrentFile.txt");
            br = new BufferedReader(fileReader);
        } catch (IOException ioException) {
            throw new IOException("File: CurrentFile.txt does not exist");
        }
        System.out.println("Successful read on current file");
        return br.readLine();
    }

    public static void updateSelectedBudget(String budgetName) throws IOException {
        try (BufferedWriter bw = new BufferedWriter(new FileWriter(System.getProperty("user.dir") + "/src/main/resources/CurrentFile.txt"))) {
            bw.write(budgetName);
            System.out.println("Current file is: " + budgetName);
        } catch (IOException ex) {
            throw new IOException("Error writing to file: " + "CurrentFile.txt");
        }
    }

    public static boolean deleteBudgetDirectory(String budgetID) {
        File targetDirectory = new File(System.getProperty("user.dir") + "/src/main/resources/" + budgetID);
        System.out.println("Deleting directory:" + targetDirectory.getPath());

        String[]entries = targetDirectory.list();
        assert entries != null;
        for(String file : entries){
            File currentFile = new File(targetDirectory.getPath(),file);
            System.out.println("Deleting file:" + currentFile.delete());
        }

        return targetDirectory.delete();
    }
    public static boolean createBudgetDirectory(String budgetID) {
        System.out.println("Directory: " + System.getProperty("user.dir") + "/src/main/resources/" + budgetID);
        File f = new File(System.getProperty("user.dir") + "/src/main/resources/" + budgetID);
        return f.mkdir();
    }
    public static void createNewIncomeFile(String budgetID, String incomeFileTitle) throws IOException {
        System.out.println("Income filepath: " + System.getProperty("user.dir") + "/src/main/resources/" + budgetID + "/" + incomeFileTitle + type);
        File incomeFile = new File(System.getProperty("user.dir") + "/src/main/resources/" + budgetID + "/" + incomeFileTitle + type);
        incomeFile.createNewFile();
    }

    public static void createNewExpenseFile(String budgetID, String expenseFileTitle) throws IOException {
        System.out.println("Expense filePath: " + System.getProperty("user.dir") + "/src/main/resources/" + budgetID + "/" + expenseFileTitle + type);
        File expenseFile = new File(System.getProperty("user.dir") + "/src/main/resources/" + budgetID + "/" + expenseFileTitle + type);
        expenseFile.createNewFile();
    }

    public static void createNewBudgetFile(String budgetID, String budgetFileTitle) throws IOException {
        System.out.println("Budget filePath: " + System.getProperty("user.dir") + "/src/main/resources/" + budgetID + "/" + budgetFileTitle + fileType);
        File budgetFile = new File(System.getProperty("user.dir") + "/src/main/resources/" + budgetID + "/" + budgetFileTitle + fileType);
        budgetFile.createNewFile();
    }

    /**
     * Method for reading (getting) a Budget from a file.
     *
     * @param fileTitle the name of the file you want to read from.
     * @return the GeneralBudget from the file.
     * @throws IOException if an input or output exception occurred.
     */
    public static GeneralBudget readGeneralBudgetFromFile(String fileTitle) throws IOException {
        GeneralBudget generalBudget = null;
        double maxAmount = 0;
        double budgetAmount = 0;
        ExpenseCategory expenseCategory = null;
        String budgetDescription = null;

        try (BufferedReader br = new BufferedReader(new FileReader( System.getProperty("user.dir") + "/" + path + readSelectedBudget() + "/" + fileTitle + fileType))) {
            String line;
            String nextLine = br.readLine();
            while ((line = nextLine) != null) {
                nextLine = br.readLine();
                if (line.startsWith(FileHandlingBudget.maxAmount)) {
                    maxAmount = Double.parseDouble(line.replace(FileHandlingBudget.maxAmount,""));
                } else if (line.startsWith(FileHandlingBudget.budgetAmount)) {
                    budgetAmount = Double.parseDouble(line.replace(FileHandlingBudget.budgetAmount,""));
                } else if (line.startsWith(FileHandlingBudget.budgetCategory)) {
                    line = line.replace(FileHandlingBudget.budgetCategory, "");
                    expenseCategory = switch (line) {
                        case "FOOD" -> ExpenseCategory.FOOD;
                        case "CLOTHES" -> ExpenseCategory.CLOTHES;
                        case "BOOKS" -> ExpenseCategory.BOOKS;
                        default -> ExpenseCategory.OTHER;
                    };
                } else if (line.startsWith(FileHandlingBudget.budgetDescription)) {
                    budgetDescription = line.replace(FileHandlingBudget.budgetDescription,"");
                }
                if (line.isEmpty() || (nextLine == null)) {
                    if(generalBudget == null){
                        generalBudget = new GeneralBudget(maxAmount);
                    } else{
                        generalBudget.addToBudget(budgetAmount,budgetDescription,expenseCategory);
                    }
                }
            }
        }
        return generalBudget;
    }
}
